#include "liblisotd_read_LISOTD.h"
#include "liblisotd_err_defs.h"

/**************************************************************************/
/*   float OffBoresiteAngle()                                             */
/*                                                                        */
/*           Calculate a pixel's off-boresite angle 'theta'               */
/*                                                                        */
/*                Last modified 8 Oct 1997 - DJB                          */
/**************************************************************************/

float OffBoresiteAngle(short x_pixel, short y_pixel, char platform[4]) {

	/*
	From: "Kevin Driscoll" <driscoll@kchange.msfc.nasa.gov>
	Date: Mon, 22 Sep 1997 13:32:54 -0500
	To: Dennis.Boccippio@msfc.nasa.gov
	Subject: pixel code
	*/
	
	/*
	; ------------------------------------------------------------------
	;  Polynomial fit of lab measurements to get pointing vector
	;  from x and y pixel location
	; ------------------------------------------------------------------
	; look_vector  = output = unit vector direction from CCD axis
	;                         to geolocated position
	; x_pixel      = input  = CCD pixel location in x direction (0-127)
	; y_pixel      = input  = CCD pixel location in y direction (0-127)
	;
	*/

    int which_data_set = 0;
	float x,y;
	float xy,convert;
	float coeff[4];
	float look_vector[3];
	float num,den,off_boresite_angle;
	float r2d=57.295779;
	float d2r=.017453293;
	
	if (!strcmp(platform,"LIS")) {
	    /* LIS optics */
	    coeff[0]= 1.4754537;
	    coeff[1]= -0.36224695;
	    coeff[2]= -0.088939824;
	    coeff[3]= -0.28203806;
	 } else if (!strcmp(platform,"OTD")) {
	    /* OTD optics */
	    coeff[0]= 1.4037314;
	    coeff[1]= 0.015948348;
	    coeff[2]=-0.86702375;
	    coeff[3]= 0.24905820;
	    } else {
			fprintf(stderr,"Error in OffBoresiteAngle()\n");	
			exit(-1);
	    }
	/*
	  x=(x_pixel-63.5)/127.;
	  y=(y_pixel-63.5)/127.;
	  y=((127-x_pixel)-63.5)/127.;
	  x=(y_pixel-63.5)/127.;
	  x=((127-x_pixel)-63.5)/127.;
	  y=(y_pixel-63.5)/127.;
	*/
	x=(x_pixel-63.5)/127.;
	y=(127-y_pixel-63.5)/127.;

	xy=sqrt(x*x+y*y);
	convert=coeff[0] + coeff[1]*xy + coeff[2]*xy*xy + coeff[3]*xy*xy*xy;

	look_vector[0]=x*convert;
	look_vector[1]=y*convert;
	look_vector[2]=sqrt(1.-(look_vector[0]*look_vector[0]+
	                      look_vector[1]*look_vector[1]));
	
	num = sqrt(look_vector[0]*look_vector[0]+look_vector[1]*look_vector[1]);
	den = look_vector[2];
	off_boresite_angle = atan2(num,den);
	
	return(r2d*off_boresite_angle);
}

/**************************************************************************/
/*   float PixelResolutionKm()                                            */
/*                                                                        */
/*        RETURN RESOLUTION OF PIXEL X,Y FROM LOOKUP TABLE                */
/*                                                                        */
/*                Last modified 08 Oct 1997 - DJB                         */
/**************************************************************************/

float PixelResolutionKm(short x, short y, char platform[4]) {

	float off_boresite, pixel_angular_res;
	float r_eff = 6370.0, z_eff, arg1 ,arg2, lam1, lam2, pixres;
	float r2d=57.295779, d2r=.017453293;
	
	off_boresite = OffBoresiteAngle(x,y,platform);
	pixel_angular_res = (OffBoresiteAngle(0,64,platform)+OffBoresiteAngle(127,64,platform))/128.0;
	
	if (!strcmp(platform,"LIS")) {
		/* LIS altitude */
		z_eff = 350.0;
	} else if (!strcmp(platform,"OTD")) {
		/* OTD altitude */
		z_eff = 740.0;
	} else {
			fprintf(stderr,"Error in PixelResolutionKm()\n");	
		exit(-1);
	}
	
	arg1 = (r_eff+z_eff)*sinf(d2r*(off_boresite-pixel_angular_res/2.0))/r_eff;
	arg2 = (r_eff+z_eff)*sinf(d2r*(off_boresite+pixel_angular_res/2.0))/r_eff;
	lam1 = PI-asinf(arg1);
	lam2 = PI-asinf(arg2);
	
	pixres = r_eff*(lam1-lam2-d2r*pixel_angular_res);
	
	return(pixres);
	
}
